@@ -396,6 +396,8 @@ def setup(dest_dir='./', branch='master', board=None, strip_sources=False):
     source_dir = os.path.join(dest_dir, OS_NAME)
     if source_dir and os.path.isdir(os.path.join(source_dir, '.git')):
         action('AliOS-Things code has been download \"%s\"' % source_dir)
+    elif source_dir and os.path.isdir(source_dir):
+        action('%s existed ...' % source_dir)
     else:
         aos_url = get_aos_url()
         aos_url += '#' + branch
@@ -405,7 +407,7 @@ def setup(dest_dir='./', branch='master', board=None, strip_sources=False):
             raise e
 
     if board and strip_sources:
-        action('Board %s specified with --strip-sources:' % board)
+        action('Board %s specified with --strip-sources ...' % board)
         _strip_sources(source_dir, board)
 
 def _strip_sources(source_dir, board):
@@ -1474,7 +1476,11 @@ def config_(var=None, value=None, global_cfg=False, unset=False, list_config=Fal
         subcommands['config'].error("too few arguments")
 
 def scons_upload(args):
-    make_args = ['scons -f ucube.py COMMAND=upload']
+    if os.path.exists('ucube.py') == True:
+        make_args = ['scons -f ucube.py COMMAND=upload']
+    else:
+        make_args = ['scons -f build/ucube.py COMMAND=upload']
+
     target_find = False
 
     for arg in args:
@@ -1522,7 +1528,7 @@ def upload(target):
 
     log("[INFO]:target: %s\n" % target)
 
-    if os.path.isdir('./include/aos') == False:
+    if os.path.isdir('./kernel/rhino') == False:
         log("[INFO]:Not in aos_sdk_path, curr_path:'%s'\n" % os.getcwd())
         aos_path = Global().get_cfg(AOS_SDK_PATH)
         if aos_path == None:
