@@ -112,6 +112,8 @@ def popen(command, stdin=None, **kwargs):
     if proc.wait() != 0:
         raise ProcessException(proc.returncode, command[0], ' '.join(list(command)), os.getcwd())
 
+    return proc.returncode
+
 
 def pquery(command, stdin=None, **kwargs):
     if very_verbose:
@@ -135,4 +137,21 @@ def pquery(command, stdin=None, **kwargs):
     if proc.returncode != 0:
         raise ProcessException(proc.returncode, command[0], ' '.join(command), os.getcwd())
 
-    return stdout
\ No newline at end of file
+    return stdout
+
+def get_country_code():
+    import requests
+    import json
+    url = "https://geoip-db.com/json"
+    try:
+        res = requests.get(url, timeout = 5)
+        data = json.loads(res.text)
+        return data['country_code']
+    except Exception:
+        return 'CN'
+
+def is_domestic():
+    if get_country_code() == 'CN':
+        return True
+    else:
+        return False
